@@ -48,7 +48,6 @@ EXPORT_NO_SYMBOLS;
 MODULE_AUTHOR("Jaroslav Kysela <perex@suse.cz>");
 MODULE_DESCRIPTION("Serial MIDI");
 MODULE_LICENSE("GPL");
-MODULE_CLASSES("{sound}");
 
 static int index[SNDRV_CARDS] = SNDRV_DEFAULT_IDX;		/* Index 0-MAX */
 static char *id[SNDRV_CARDS] = SNDRV_DEFAULT_STR;		/* ID for this card */
@@ -63,25 +62,18 @@ static int boot_devs;
 
 module_param_array(index, int, boot_devs, 0444);
 MODULE_PARM_DESC(index, "Index value for serial device.");
-MODULE_PARM_SYNTAX(index, SNDRV_INDEX_DESC);
 module_param_array(id, charp, boot_devs, 0444);
 MODULE_PARM_DESC(id, "ID string for serial device.");
-MODULE_PARM_SYNTAX(id, SNDRV_ID_DESC);
 module_param_array(enable, bool, boot_devs, 0444);
 MODULE_PARM_DESC(enable, "Enable serial device.");
-MODULE_PARM_SYNTAX(enable, SNDRV_ENABLE_DESC);
 module_param_array(sdev, charp, boot_devs, 0444);
 MODULE_PARM_DESC(sdev, "Device file string for serial device.");
-MODULE_PARM_SYNTAX(sdev, SNDRV_ID_DESC);
 module_param_array(speed, int, boot_devs, 0444);
 MODULE_PARM_DESC(speed, "Speed in bauds.");
-MODULE_PARM_SYNTAX(speed, SNDRV_ENABLED ",allows:{9600,19200,38400,57600,115200},dialog:list");
 module_param_array(adaptor, int, boot_devs, 0444);
 MODULE_PARM_DESC(adaptor, "Type of adaptor.");
-MODULE_PARM_SYNTAX(adaptor, SNDRV_ENABLED ",allows:{{0=Soundcanvas,1=MS-124T,2=MS-124W S/A,3=MS-124W M/B}},dialog:list");
 module_param_array(outs, int, boot_devs, 0444);
 MODULE_PARM_DESC(outs, "Number of MIDI outputs.");
-MODULE_PARM_SYNTAX(outs, SNDRV_ENABLED ",allows:{{1,16}},dialog:list");
 module_param_array(devices, int, boot_devs, 0444);
 MODULE_PARM_DESC(devices, "Number of devices to attach to the card.");
 module_param_array(handshake, int, boot_devs, 0444);
@@ -297,7 +289,7 @@ static int close_tty(serialmidi_t *serial, unsigned int mode)
 
 static void ldisc_receive_buf(struct tty_struct *tty, const unsigned char *cp, char *fp, int count)
 {
-	serialmidi_t *serial = snd_magic_cast(serialmidi_t, tty->disc_data, return);
+	serialmidi_t *serial = tty->disc_data;
 
 	if (serial == NULL)
 		return;
@@ -345,14 +337,14 @@ static void tx_loop(serialmidi_t *serial)
 
 static void ldisc_write_wakeup(struct tty_struct *tty)
 {
-	serialmidi_t *serial = snd_magic_cast(serialmidi_t, tty->disc_data, return);
+	serialmidi_t *serial = tty->disc_data;
 
 	tx_loop(serial);
 }
 
 static void snd_serialmidi_output_trigger(snd_rawmidi_substream_t * substream, int up)
 {
-	serialmidi_t *serial = snd_magic_cast(serialmidi_t, substream->rmidi->private_data, return);
+	serialmidi_t *serial = substream->rmidi->private_data;
 	
 	if (up) {
 		set_bit(SERIAL_MODE_BIT_OUTPUT_TRIGGERED, &serial->mode);
@@ -366,7 +358,7 @@ static void snd_serialmidi_output_trigger(snd_rawmidi_substream_t * substream, i
 
 static void snd_serialmidi_input_trigger(snd_rawmidi_substream_t * substream, int up)
 {
-	serialmidi_t *serial = snd_magic_cast(serialmidi_t, substream->rmidi->private_data, return);
+	serialmidi_t *serial = substream->rmidi->private_data;
 
 	if (up) {
 		set_bit(SERIAL_MODE_BIT_INPUT_TRIGGERED, &serial->mode);
@@ -377,7 +369,7 @@ static void snd_serialmidi_input_trigger(snd_rawmidi_substream_t * substream, in
 
 static int snd_serialmidi_output_open(snd_rawmidi_substream_t * substream)
 {
-	serialmidi_t *serial = snd_magic_cast(serialmidi_t, substream->rmidi->private_data, return -ENXIO);
+	serialmidi_t *serial = substream->rmidi->private_data;
 	int err;
 
 	if ((err = open_tty(serial, SERIAL_MODE_BIT_OUTPUT)) < 0)
@@ -388,7 +380,7 @@ static int snd_serialmidi_output_open(snd_rawmidi_substream_t * substream)
 
 static int snd_serialmidi_output_close(snd_rawmidi_substream_t * substream)
 {
-	serialmidi_t *serial = snd_magic_cast(serialmidi_t, substream->rmidi->private_data, return -ENXIO);
+	serialmidi_t *serial = substream->rmidi->private_data;
 
 	serial->substream_output = NULL;
 	return close_tty(serial, SERIAL_MODE_BIT_OUTPUT);
@@ -396,7 +388,7 @@ static int snd_serialmidi_output_close(snd_rawmidi_substream_t * substream)
 
 static int snd_serialmidi_input_open(snd_rawmidi_substream_t * substream)
 {
-	serialmidi_t *serial = snd_magic_cast(serialmidi_t, substream->rmidi->private_data, return -ENXIO);
+	serialmidi_t *serial = substream->rmidi->private_data;
 	int err;
 
 	if ((err = open_tty(serial, SERIAL_MODE_BIT_INPUT)) < 0)
@@ -407,7 +399,7 @@ static int snd_serialmidi_input_open(snd_rawmidi_substream_t * substream)
 
 static int snd_serialmidi_input_close(snd_rawmidi_substream_t * substream)
 {
-	serialmidi_t *serial = snd_magic_cast(serialmidi_t, substream->rmidi->private_data, return -ENXIO);
+	serialmidi_t *serial = substream->rmidi->private_data;
 
 	serial->substream_input = NULL;
 	return close_tty(serial, SERIAL_MODE_BIT_INPUT);
@@ -431,13 +423,13 @@ static int snd_serialmidi_free(serialmidi_t *serial)
 {
 	if (serial->sdev);
 		kfree(serial->sdev);
-	snd_magic_kfree(serial);
+	kfree(serial);
 	return 0;
 }
 
 static int snd_serialmidi_dev_free(snd_device_t *device)
 {
-	serialmidi_t *serial = snd_magic_cast(serialmidi_t, device->device_data, return -ENXIO);
+	serialmidi_t *serial = device->device_data;
 	return snd_serialmidi_free(serial);
 }
 
@@ -490,7 +482,7 @@ static int __init snd_serialmidi_create(snd_card_t *card, const char *sdev,
 	if (outs > 16)
 		outs = 16;
 
-	if ((serial = snd_magic_kcalloc(serialmidi_t, 0, GFP_KERNEL)) == NULL)
+	if ((serial = kcalloc(1, sizeof(*serial), GFP_KERNEL)) == NULL)
 		return -ENOMEM;
 
 	init_MUTEX(&serial->open_lock);
