@@ -40,6 +40,7 @@ struct cond {
 	struct dep *dep;	/* dependency pointer */
 	int left;		/* left brackets */
 	int right;		/* right brackets (after this condition element) */
+	int not;
 	int type;
 	struct cond *next;
 	struct cond *stack_prev;
@@ -100,6 +101,7 @@ static char *kernel_deps[] = {
 	"EISA",
 	"PCI",
 	"SBUS",
+	"I2C",
 	"L3",
 	"USB",
 	"PCMCIA",
@@ -112,6 +114,7 @@ static char *kernel_deps[] = {
 	"PPC64",
 	"X86_64",
 	"X86",
+	"MIPS64",
 	"IA32_EMULATION",
 	/* architecture specific */
 	"ARCH_SA1100",
@@ -125,16 +128,15 @@ static char *kernel_deps[] = {
 	NULL
 };
 
-/* @ -> add to output for all cards */
 /* % -> always true */
 static char *no_cards[] = {
 	"%SOUND",
 	"SOUND_PRIME",
-	"%@SND",
-	"@SND_TIMER",
-	"@SND_HWDEP",
-	"@SND_RAWMIDI",
-	"@SND_PCM",
+	"%SND",
+	"SND_TIMER",
+	"SND_HWDEP",
+	"SND_RAWMIDI",
+	"SND_PCM",
 	"SND_SEQUENCER",
 	"SND_MIXER_OSS",
 	"SND_PCM_OSS",
@@ -146,16 +148,19 @@ static char *no_cards[] = {
 	"SND_DEBUG_DETECT",
 	"SND_VERBOSE_PRINTK",
 	"SND_BIT32_EMUL",
-	"@SND_OPL3_LIB",
-	"@SND_OPL4_LIB",
-	"@SND_VX_LIB",
-	"@SND_MPU401_UART",
+	"SND_OPL3_LIB",
+	"SND_OPL4_LIB",
+	"SND_VX_LIB",
+	"SND_MPU401_UART",
+	"SND_GUS_SYNTH",
+	"SND_AC97_CODEC",
 	NULL
 };
 
 #define READ_STATE_NONE		0
 #define READ_STATE_CONFIG	1
 #define READ_STATE_MENU		2
+#define READ_STATE_COMMENT	3
 
 static void nomem(void)
 {
@@ -275,6 +280,9 @@ static int read_file_1(const char *filename, struct cond **template)
 			if (result < 0)
 				goto __end;
 		}
+		if (!strncmp(buffer, "comment", 7)) {
+			state = READ_STATE_COMMENT;
+		}
 		switch (state) {
 		case READ_STATE_CONFIG:
 			if (!strncmp(buffer, "\ttristate ", 10))
@@ -299,6 +307,7 @@ static int read_file_1(const char *filename, struct cond **template)
 				(*template)->dep = ntemplate->dep;
 				(*template)->left = ntemplate->left;
 				(*template)->right = ntemplate->right;
+				(*template)->not = ntemplate->not;
 				(*template)->type = ntemplate->type;
 				(*template)->next = ntemplate->next;
 				free(ntemplate);
@@ -355,6 +364,11 @@ static struct cond * create_cond(char *line)
 				word[i - 1] = word[i];
 			cond->left++;
 		}
+		/* hack for !XXX */
+		if (word[0] == '!' && isascii(word[1])) {
+			cond->not = 1;
+			memmove(word, word + 1, strlen(word));
+		}
 		for (i = 0; i < strlen(word); i++) {
 			if (word[i] == '!' || word[i] == '=' ||
 			    word[i] == '&' || word[i] == ')') {
@@ -753,8 +767,6 @@ static int is_toplevel(struct dep *dep)
 		str = no_cards[idx];
 		if (*str == '%')
 			str++;
-		if (*str == '@')
-			str++;
 		if (!strcmp(str, dep->name))
 			return 0;
 	}
@@ -774,29 +786,6 @@ static int is_always_true(struct dep *dep)
 		if (*str != '%')
 			continue;
 		str++;
-		if (*str == '@')
-			str++;
-		if (!strcmp(str, dep->name))
-			return 1;
-	}
-	return 0;
-}
-
-// is CONFIG_ variable belongs to all card output
-static int belongs_to_all(struct dep *dep)
-{
-	int idx;
-	char *str;
-
-	if (dep == NULL)
-		return 0;
-	for (idx = 0; no_cards[idx]; idx++) {
-		str = no_cards[idx];
-		if (*str == '%')
-			str++;
-		if (*str != '@')
-			continue;
-		str++;
 		if (!strcmp(str, dep->name))
 			return 1;
 	}
@@ -804,7 +793,7 @@ static int belongs_to_all(struct dep *dep)
 }
 
 // Print out ALL deps for firstdep (Cards, Deps)
-static void output_card_list(struct dep *firstdep, int space, int size)
+static void output_card_list(struct dep *firstdep, int space, int size, int tristate)
 {
 	struct dep *temp_dep=firstdep;
 	char *card_name;
@@ -816,7 +805,9 @@ static void output_card_list(struct dep *firstdep, int space, int size)
 	while(temp_dep) {
 		if (!is_toplevel(temp_dep))
 			goto __skip;
-		if (temp_dep->is_bool)
+		if (tristate && temp_dep->is_bool)
+			goto __skip;
+		if (!tristate && !temp_dep->is_bool)
 			goto __skip;
 		card_name=get_card_name(temp_dep->name);
 		if (card_name) {
@@ -871,10 +862,8 @@ static void sel_print_acinclude(struct sel *sel)
 		sel_print_acinclude(nsel);
 		if (!nsel->dep->hitflag) {
 			nsel->dep->hitflag = 1;
-			printf("\t\tCONFIG_%s=\"%c\"\n", nsel->name,
+			printf("      CONFIG_%s=\"%c\"\n", nsel->name,
 				(nsel->dep && nsel->dep->is_bool) ? 'y' : 'm');
-			printf("\t\tAC_DEFINE(CONFIG_%s%s)\n", nsel->name,
-				(nsel->dep && nsel->dep->is_bool) ? "" : "_MODULE");
 		}
 	}
 }
@@ -901,30 +890,44 @@ static void output_acinclude(void)
 	}
 	printf("])\n\n");
 
-	printf("AC_DEFUN([ALSA_TOPLEVEL_ALL], [\n");
+	printf("AC_DEFUN([ALSA_TOPLEVEL_SELECT], [\n");
+	printf("dnl Check for which cards to compile driver for...\n");
+	printf("AC_MSG_CHECKING(for which soundcards to compile driver for)\n");
+	printf("AC_ARG_WITH(cards,\n"
+	       "  [  --with-cards=<list>     compile driver for cards in <list>; ]\n"
+	       "  [                        cards may be separated with commas; ]\n"
+	       "  [                        'all' compiles all drivers; ]\n"
+	       "  [                        Possible cards are: ]\n");
+	output_card_list(all_deps, 24, 50, 1);
+	printf(" ]\n");
+	printf("  [                        Possible additional options are: ]\n");
+	output_card_list(all_deps, 24, 50, 0);
+	printf(" ],\n");
+	printf("  cards=\"$withval\", cards=\"all\")\n");
+
+	/* check cards */
+	printf("cards=`echo $cards | sed 's/,/ /g'`\n");
+	printf("for card in $cards; do\n"
+	       "  probed=\n");
 	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
-		if (!belongs_to_all(tempdep))
-			continue;
-		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
-		if (!strncmp(text, "CONFIG_SND", 10)) {
-			printf("\t%s=\"%c\"\n", text,
-			       tempdep->is_bool ? 'y' : 'm');
-			printf("\tAC_DEFINE(%s%s)\n", text,
-			       tempdep->is_bool ? "" : "_MODULE");
-		}
-		free(text);
-	}
-	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
-		int put_if = 0;
+		int put_if;
 		if (!is_toplevel(tempdep))
 			continue;
+		text = get_card_name(tempdep->name);
+		if (! text)
+			continue;
+		printf("  if test \"$card\" = \"all\" -o \"$card\" = \"%s\"; then\n", text);
+		free(text);
+		put_if = 0;
 		for (cond = tempdep->cond, cond_prev = NULL; cond; cond = cond->next) {
 			if (!put_if)
-				printf("\tif ");
+				printf("    if ");
 			else {
 				printf(cond_prev->type == COND_AND ? " &&" : " ||");
-				printf("\n\t   ");
+				printf("\n      ");
 			}
+			if (cond->not)
+				printf(" ! ");
 			for (j = 0; j < cond->left; j++)
 				printf("(");
 			printf("( test \"$CONFIG_%s\" == \"y\" -o \"$CONFIG_%s\" == \"m\" )", cond->name, cond->name);
@@ -935,90 +938,47 @@ static void output_acinclude(void)
 		}
 		if (put_if)
 			printf("; then\n");
+
+		sel_remove_hitflags();
+		for (sel = tempdep->sel; sel; sel = sel->next)
+			sel_print_acinclude(sel);			
+		for (sel = tempdep->sel; sel; sel = sel->next) {
+			if (is_always_true(sel->dep))
+				continue;
+			printf("      CONFIG_%s=\"%c\"\n", sel->name,
+			       (sel->dep && sel->dep->is_bool) ? 'y' : 'm');
+		}
 		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
-		printf("\t");
-		if (put_if)
-			printf("  ");
-		printf("%s=\"%c\"\n", text,
-		       tempdep->is_bool ? 'y' : 'm');
-		printf("\t");
-		if (put_if)
-			printf("  ");
-		printf("AC_DEFINE(%s%s)\n", text,
-		       tempdep->is_bool ? "" : "_MODULE");
-		if (put_if)
-			printf("\tfi\n");
+		printf("      %s=\"%c\"\n", text, tempdep->is_bool ? 'y' : 'm');
 		free(text);
+		printf("      probed=1\n");
+		if (put_if)
+			printf("    elif test -z \"$probed\"; then\n"
+			       "      probed=0\n"
+			       "    fi\n");
+		printf("  fi\n");
 	}
-	printf("])\n\n");
-	
-	printf("AC_DEFUN([ALSA_TOPLEVEL_SELECT], [\n");
-	printf("dnl Check for which cards to compile driver for...\n");
-	printf("AC_MSG_CHECKING(for which soundcards to compile driver for)\n");
-	printf("AC_ARG_WITH(cards,\n\
-  [  --with-cards=<list>     compile driver for cards in <list>; ]\n\
-  [                        cards may be separated with commas; ]\n\
-  [                        'all' compiles all drivers; ]\n\
-  [                        Possible cards are: ]\n");
-	output_card_list(all_deps, 24, 50);
-	printf(" ],\n");
-	printf("  cards=\"$withval\", cards=\"all\")\n");
-	printf("if test \"$cards\" = \"all\"; then\n");
-	printf("  ALSA_TOPLEVEL_ALL\n");
-	printf("  AC_MSG_RESULT(all)\n");
-	printf("else\n");
-	printf("  cards=`echo $cards | sed 's/,/ /g'`\n");
-	printf("  for card in $cards\n");
-	printf("  do\n");
-	printf("    case \"$card\" in\n");
+	printf("  if test -z \"$probed\"; then\n"
+	       "    AC_MSG_ERROR(Unknown soundcard $card)\n"
+	       "  elif test \"$probed\" = \"0\"; then\n"
+	       "    AC_MSG_ERROR(Unsupported soundcard $card)\n"
+	       "  fi\n"
+	       "done\n\n");
+	printf("AC_MSG_RESULT($cards)\n\n");
+	printf("CONFIG_SND=\"m\"\n");
 	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
 		if (!is_toplevel(tempdep))
 			continue;
-		text = get_card_name(tempdep->name);
-		if (text) {
-			printf("\t%s)\n", text);
-			free(text);
-			printf("\t\tCONFIG_SND=\"m\"\n");
-			printf("\t\tAC_DEFINE(CONFIG_SND_MODULE)\n");
-			sel_remove_hitflags();
-			for (sel = tempdep->sel; sel; sel = sel->next)
-				sel_print_acinclude(sel);			
-#if 0
-			for (cond = tempdep->cond; cond; cond = cond->next) {
-				if (is_always_true(cond->dep) &&
-				    strcmp(cond->name, "SND"))
-					continue;
-				printf("\t\tCONFIG_%s=\"%c\"\n", cond->name,
-				       (cond->dep && cond->dep->is_bool) ? 'y' : 'm');
-				printf("\t\tAC_DEFINE(CONFIG_%s%s)\n", cond->name,
-				       (cond->dep && cond->dep->is_bool) ? "" : "_MODULE");
-			}
-#endif
-			for (sel = tempdep->sel; sel; sel = sel->next) {
-				if (is_always_true(sel->dep))
-					continue;
-				printf("\t\tCONFIG_%s=\"%c\"\n", sel->name,
-				       (sel->dep && sel->dep->is_bool) ? 'y' : 'm');
-				printf("\t\tAC_DEFINE(CONFIG_%s%s)\n", sel->name,
-				       (sel->dep && sel->dep->is_bool) ? "" : "_MODULE");
-			}
-			text = convert_to_config_uppercase("CONFIG_", tempdep->name);
-			printf("\t\t%s=\"%c\"\n", text,
-			       tempdep->is_bool ? 'y' : 'm');
-			printf("\t\tAC_DEFINE(%s%s)\n", text,
-			       tempdep->is_bool ? "" : "_MODULE");
-			printf("\t\t;;\n");
-			free(text);
-		}
+		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
+		printf("if test -n \"$%s\"; then\n", text);
+		if (tempdep->is_bool)
+			printf("  AC_DEFINE(%s)\n", text);
+		else
+			printf("  AC_DEFINE(%s_MODULE)\n", text);
+		printf("fi\n");
+		free(text);
 	}
-	printf("\t*)\n");
-	printf("\t\techo \"Unknown soundcard $card, exiting!\"\n");
-	printf("\t\texit 1\n");
-	printf("\t\t;;\n");
-	printf("    esac\n");
-	printf("  done\n");
-	printf("  AC_MSG_RESULT($cards)\n");
-	printf("fi\n");
+
 	printf("])\n\n");
 	printf("AC_DEFUN([ALSA_TOPLEVEL_OUTPUT], [\n");
 	printf("dnl output all subst\n");
@@ -1249,6 +1209,6 @@ static void usage(char *programname)
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Options:\n");
 	fprintf(stderr, "       --basedir <basedir>\n");
-	fprintf(stderr, "       --hidendir <hidendir>\n");
+	fprintf(stderr, "       --hiddendir <hiddendir>\n");
 	exit(EXIT_FAILURE);
 }
